@@ -4,15 +4,16 @@ use strict;
 use Config;
 use File::Spec;
 use File::Basename;
+use IPC::Open3 'open3';
 
 sub WritePugs {
     my $self = shift;
-    
+
     my $install_version = shift;
     die "Install version must be 5 or 6 for WritePugs"
       unless $install_version =~ /^[56]$/;
 
-    $self->setup_perl6_install 
+    $self->setup_perl6_install
       if $install_version eq '6';
 
     $self->set_blib($install_version);
@@ -52,7 +53,7 @@ sub is_extension_build {
 
 sub set_blib {
     my $self = shift;
-    my $perl_version = shift 
+    my $perl_version = shift
       or die "Must pass Perl version (5 or 6)";
     my $base = $self->{_top}{base};
     my $blib = ($perl_version == 5 || $self->is_extension_build)
@@ -62,18 +63,20 @@ sub set_blib {
       : die "Perl version '$perl_version' is bad. Must be 5 or 6.";
     my $path = File::Spec->catdir($base, $blib);
 
-    $self->makemaker_args->{INST_LIB} = 
+    $self->makemaker_args->{INST_LIB} =
       File::Spec->catfile($path, "lib");
-    $self->makemaker_args->{INST_ARCHLIB} = 
+    $self->makemaker_args->{INST_ARCHLIB} =
       File::Spec->catfile($path, "arch");
-    $self->makemaker_args->{INST_SCRIPT} = 
+    $self->makemaker_args->{INST_SCRIPT} =
       File::Spec->catfile($path, "script");
-    $self->makemaker_args->{INST_BIN} = 
+    $self->makemaker_args->{INST_BIN} =
       File::Spec->catfile($path, "bin");
-    $self->makemaker_args->{INST_MAN1DIR} = 
+    $self->makemaker_args->{INST_MAN1DIR} =
       File::Spec->catfile($path, "man1");
-    $self->makemaker_args->{INST_MAN3DIR} = 
+    $self->makemaker_args->{INST_MAN3DIR} =
       File::Spec->catfile($path, "man3");
+    $self->makemaker_args->{MAN1PODS} = {};
+    $self->makemaker_args->{MAN3PODS} = {};
     $self->{MM}{INST_AUTODIR} = '$(INST_LIB)/$(BASEEXT)';
     $self->{MM}{INST_ARCHAUTODIR} = '$(INST_ARCHLIB)/$(FULLEXT)';
 }
@@ -100,9 +103,9 @@ sub pugs_fix_makefile {
     my $full_blib = File::Spec->catfile($base, 'blib6', 'lib');
     open MAKEFILE, '< Makefile' or die $!;
     my $makefile = do { local $/; <MAKEFILE> };
-    $full_pugs =~ s{\\}{\\\\}g; 
+    $full_pugs =~ s{\\}{\\\\}g;
     $full_pugs =~ s{'}{\\'}g;
-    $full_blib =~ s{\\}{\\\\}g; 
+    $full_blib =~ s{\\}{\\\\}g;
     $full_blib =~ s{'}{\\'}g;
     $makefile =~ s/\b(runtests \@ARGV|test_harness\(\$\(TEST_VERBOSE\), )/ENV->{HARNESS_PERL} = q{$full_pugs}; \@ARGV = grep !\/[A-Z]\/, map glob, \@ARGV; ENV->{PERL6LIB} = q{$full_blib}; $1/;
     $makefile =~ s!("-MExtUtils::Command::MM")!"-I../../inc" "-I../inc" "-Iinc" $1!g;
@@ -155,19 +158,24 @@ sub assert_ghc {
 *** Please install a newer version from http://haskell.org/ghc/.
 .
     }
-    my $ghc_flags = "-H200m -L. -Lsrc -Lsrc/pcre -I. -Isrc -Isrc/pcre ";
-    $ghc_flags .= " -i. -isrc -isrc/pcre -static "; 
-    $ghc_flags .= " -Wall "
+    my $ghc_flags = "-H0 -L. -Lsrc -Lsrc/pcre -I. -Isrc -Isrc/pcre ";
+    $ghc_flags .= " -i. -isrc -isrc/pcre -static ";
+    $ghc_flags .= " -Wall -Werror "
       unless $self->is_extension_build;
     $ghc_flags .= " -fno-warn-missing-signatures -fno-warn-name-shadowing ";
     $ghc_flags .= " -I../../src -i../../src "
       if $self->is_extension_build;
-    $ghc_flags .= " -I$Config{archlib}/CORE -L$Config{archlib}/CORE -i$Config{archlib}/CORE -lperl" 
+    $ghc_flags .= " -I$Config{archlib}/CORE -L$Config{archlib}/CORE -i$Config{archlib}/CORE -lperl"
       if $ENV{PUGS_EMBED} and $ENV{PUGS_EMBED} =~ /perl5/i;
     #$ghc_flags .= " -fno-warn-deprecations -fno-warn-orphans";
     return ($ghc, $ghc_version, $ghc_flags);
 }
 
+sub has_ghc_package {
+    my ($self, $package) = @_;
+    `ghc-pkg describe $package` =~ /package-url/;
+}
+
 sub fixpaths {
     my $self = shift;
     my $text = shift;
