@@ -3,7 +3,7 @@
 module Compile.Haskell where
 
 #undef PUGS_HAVE_TH
-#include "pugs_config.h"
+#include "../pugs_config.h"
 #ifndef PUGS_HAVE_TH
 genGHC = error "Template Haskell support not compiled in"
 #else
